@@ -13,36 +13,52 @@ import sys
 import json
 
 default_stage = 'dev'
-fresh_notes = '''
-A skeleton of the new lambda, {}, has been created.
+fresh_notes = '''A skeleton of the new lambda, {}, has been created.
 
 In {}/{}/config you will find a config.ini file that you should
 fill in with parameters for your own account.
 
 Develop the lambda function as needed then you can deploy it with:
-lambdatool deploy.
+lambdatool deploy. The lambda has been started in main.py.
 '''
 
 
 @click.group()
-@click.version_option(version='0.0.2')
+@click.version_option(version='0.3.2')
 def cli():
     pass
 
 
 @cli.command()
-@click.option('-d', '--directory')
-@click.option('-n', '--name', required=True)
-def new(directory, name):
+@click.option('-d', '--directory', help='target directory for new Lambda, defaults to current directory')
+@click.option('-n', '--name', help='name of the new lambda skeleton', required=True)
+@click.option('-s', '--service', help='create a flask like micro-service', is_flag=True)
+@click.option('-p', '--profile', help='AWS CLI profile to use in the deployment, more details at http://docs.aws.amazon.com/cli/latest/userguide/cli-chap-getting-started.html')
+@click.option('-r', '--region', help='target region, defaults to your credentials default region')
+def new(directory, name, service, profile, region):
     command_line = {}
     command_line['name'] = name
-    command_line['template_directory'] = '{}/template'.format(lambdatool.__path__[0])
+
+    if service:
+        command_line['template_directory'] = '{}/template/service'.format(lambdatool.__path__[0])
+    else:
+        command_line['template_directory'] = '{}/template/simple'.format(lambdatool.__path__[0])
 
     if directory:
         command_line['directory'] = directory
     else:
         command_line['directory'] = '.'
 
+    if profile:
+        command_line['profile'] = profile
+    else:
+        command_line['profile'] = None
+
+    if region:
+        command_line['region'] = region
+    else:
+        command_line['region'] = None
+
     if start_new_lambda(command_line):
         sys.exit(0)
     else:
@@ -50,10 +66,10 @@ def new(directory, name):
 
 
 @cli.command()
-@click.option('-d', '--directory')
-@click.option('-s', '--stage')
-@click.option('-p', '--profile')
-@click.option('-r', '--region')
+@click.option('-d', '--directory', help='scratch directory for deploy, defaults to /tmp')
+@click.option('-s', '--stage', help='environment/stage used to name and deploy the Lambda function, defaults to dev')
+@click.option('-p', '--profile', help='AWS CLI profile to use in the deployment, more details at http://docs.aws.amazon.com/cli/latest/userguide/cli-chap-getting-started.html')
+@click.option('-r', '--region', help='target region, defaults to your credentials default region')
 def deploy(directory, stage, profile, region):
     command_line = {}
 
@@ -94,6 +110,8 @@ def start_new_lambda(command_line):
 
     if tool.create_lambda():
         logging.info('create_new_lambda() went well')
+        print('\n\n\n\n')
+        print('********************************************************************************')
         print(fresh_notes.format(
             command_line['name'],
             command_line['directory'],
@@ -112,6 +130,7 @@ def deploy_lambda(command_line):
 
     if tool.deploy_lambda():
         logging.info('deploy_lambda() went well')
+        return True
     else:
         logging.error('deploy_lambda() did not go well')
         sys.exit(1)
