@@ -11,6 +11,7 @@ import shutil
 import zipfile
 import utility #noqa
 import ConfigParser
+from stack_tool import StackTool
 from template_creator import TemplateCreator
 from stackility import CloudStackUtility
 
@@ -52,8 +53,10 @@ class LambdaDeployer:
     _lambda_name = None
     _hash = None
     _tag_file = None
-    _stack_properties_file = None
+    _stack_properties = {}
     _template_directory = None
+    _s3_client = None
+    _ssm_client = None
 
     def __init__(self, config_block):
         """
@@ -81,6 +84,7 @@ class LambdaDeployer:
             self._profile = config_block['profile']
             self._region = config_block['region']
             self._template_directory = config_block['template_directory']
+            self._init_boto3_clients()
         else:
             logging.error('config block was garbage')
             raise SystemError
@@ -177,27 +181,44 @@ class LambdaDeployer:
 
     def create_stack(self):
         try:
-            command_line = {}
-            command_line['stackName'] = 'lambda-{}-{}'.format(
-                self._lambda_name,
-                self._stage
+            ini_data = {}
+            ini_data['environment'] = {}
+            ini_data['tags'] = {}
+            ini_data['parameters'] = {}
+
+            stack_name = '{}-lambda-{}'.format(
+                self._stage,
+                self._lambda_name
             )
 
-            command_line['destinationBucket'] = self._ini_data.get(self._stage, {}).get('bucket', None)
-            command_line['templateFile'] = '{}/template.yaml'.format(self._work_directory)
-            command_line['tagFile'] = self._tag_file
-            command_line['yaml'] = True
-            command_line['dryrun'] = False
-            command_line['parameterFile'] = self._stack_properties_file
-            command_line['codeVersion'] = self._hash
-            command_line['profile'] = self._profile
-            command_line['region'] = self._region
+            tmp_env = self._ini_data[self._stage]
+            ini_data['environment']['template'] = '{}/template.yaml'.format(self._work_directory)
+            ini_data['environment']['bucket'] = tmp_env['bucket']
+            ini_data['environment']['stack_name'] = stack_name
+            ini_data['codeVersion'] = self._hash
+            if self._region:
+                ini_data['environment']['region'] = self._region
 
-            stack_driver = CloudStackUtility(command_line)
+            if self._profile:
+                ini_data['environment']['profile'] = self._profile
+
+            ini_data['tags'] = {'tool': 'lambdatool'}
+            ini_data['parameters'] = self._stack_properties
+            ini_data['yaml'] = True
+
+            stack_driver = CloudStackUtility(ini_data)
             if stack_driver.upsert():
                 logging.info('stack create/update was started successfully.')
                 if stack_driver.poll_stack():
                     logging.info('stack create/update was finished successfully.')
+                    st = StackTool(
+                        stack_name,
+                        self._stage,
+                        self._profile,
+                        self._region,
+                        self._cf_client,
+                    )
+                    st.print_stack_info()
                     return True
                 else:
                     logging.error('stack create/update was did not go well.')
@@ -216,13 +237,12 @@ class LambdaDeployer:
                     self._work_directory,
                     self._stage
             )
-            stack_properties = '{}/stack.properties'.format(self._work_directory)
             output_file = '{}/template.yaml'.format(self._work_directory)
             template_file = '{}/template_template'.format(self._template_directory)
-            templateCreator = TemplateCreator()
+            templateCreator = TemplateCreator(self._ssm_client)
             template_created = templateCreator.create_template(
                 function_properties=function_properties,
-                stack_properties=stack_properties,
+                stack_properties=self._stack_properties,
                 output_file=output_file,
                 template_file=template_file,
                 region=self._region,
@@ -239,11 +259,6 @@ class LambdaDeployer:
 
     def create_stack_properties(self):
         try:
-            self._stack_properties_file = '{}/stack.properties'.format(
-                self._work_directory
-            )
-
-            logging.info('creating stack properties file: {}'.format(self._stack_properties_file))
             bucket = self._ini_data.get(self._stage, {}).get('bucket', None)
             memory_size = self._ini_data.get(self._stage, {}).get('memory', '128')
             role = self._ini_data.get(self._stage, {}).get('role', None)
@@ -255,34 +270,36 @@ class LambdaDeployer:
             lambda_schedule_expression = self._ini_data.get(self._stage, {}).get('scheduleexpression', None)
             service = self._ini_data.get(self._stage, {}).get('service', None)
 
-            with open(self._stack_properties_file, "w") as outfile:
-                outfile.write('s3Bucket={}\n'.format(bucket))
-                outfile.write('s3Key={}\n'.format(self._package_key))
-                outfile.write('functionName={}-{}\n'.format(
-                        self._lambda_name,
-                        self._stage
-                    )
-                )
-                outfile.write('handler=main.lambda_handler\n')
-                outfile.write('runTime={}\n'.format(PYTHON))
-                outfile.write('memorySize={}\n'.format(memory_size))
-                outfile.write('role={}\n'.format(role))
-                outfile.write('timeOut={}\n'.format(timeout))
-                outfile.write('securityGroupIds={}\n'.format(security_group))
-                outfile.write('subnetIds={}\n'.format(subnets))
+            wrk = {}
+            wrk['s3Bucket'] = bucket
+            wrk['s3Key'] = self._package_key
+            wrk['functionName'] = '{}-{}'.format(self._lambda_name, self._stage)
+            wrk['handler'] = 'main.lambda_handler'
+            wrk['runTime'] = PYTHON
+            wrk['memorySize'] = memory_size
+            wrk['role'] = role
+            wrk['timeOut'] = timeout
+            wrk['securityGroupIds'] = security_group
+            wrk['subnetIds'] = subnets
 
-                if sns_topic_arn:
-                    outfile.write('snsTopicARN={}\n'.format(sns_topic_arn))
+            if sns_topic_arn:
+                logging.info('subscribing lambda to SNS topic: {}'.format(sns_topic_arn))
+                wrk['snsTopicARN'] = sns_topic_arn
 
-                if trusted_service:
-                    outfile.write('trustedService={}\n'.format(trusted_service))
+            if trusted_service:
+                logging.info('the lambda will trust: {}'.format(trusted_service))
+                wrk['trustedService'] = trusted_service
 
-                if lambda_schedule_expression:
-                    outfile.write('scheduleExpression={}\n'.format(lambda_schedule_expression))
+            if lambda_schedule_expression:
+                logging.info('the lambda will be scheduled by: {}'.format(lambda_schedule_expression))
+                wrk['scheduleExpression'] = lambda_schedule_expression
 
-                if service:
-                    if service.lower() == 'true':
-                        outfile.write('service=true\n')
+            if service:
+                if service.lower() == 'true':
+                    logging.info('the lambda will integrated with an API gateway')
+                    wrk['service'] = 'true'
+
+            self._stack_properties = wrk
 
             return True
         except Exception as x:
@@ -323,14 +340,8 @@ class LambdaDeployer:
             if not self._region:
                 self._region = boto3.session.Session().region_name
 
-            s3_client = utility.get_api_client(
-                self._profile,
-                self._region,
-                's3'
-            )
-
             bucket = self._ini_data.get(self._stage, {}).get('bucket', None)
-            if s3_client:
+            if self._s3_client:
                 logging.info('S3 client allocated')
                 logging.info('preparing upload to s3://{}/{}'.format(bucket, self._package_key))
             else:
@@ -339,7 +350,7 @@ class LambdaDeployer:
 
             if bucket:
                 with open(self._package_name, 'rb') as the_package:
-                    s3_client.upload_fileobj(the_package, bucket, self._package_key)
+                    self._s3_client.upload_fileobj(the_package, bucket, self._package_key)
             else:
                 logging.error('S3 bucket not found in config/config.ini')
                 return False
@@ -378,25 +389,36 @@ class LambdaDeployer:
             return False
 
     def set_hash(self):
+        random_bits = []
+        random_bits.append((str(uuid.uuid4()))[:8])
+        random_bits.append((str(uuid.uuid4()))[:8])
+
         try:
             repo = git.Repo(search_parent_directories=False)
             hash = repo.head.object.hexsha
             hash = hash[:8]
-            self._hash = hash
-            return True
         except Exception:
-            return False
+            hash = random_bits[1]
+
+        self._hash = '{}-{}'.format(hash, random_bits[0])
+        return True
 
     def verify_lambda_directory(self):
-
         return os.path.isfile(DEFAULT_MODULE_FILE)
 
     def find_lambda_name(self):
+        lambdatool = '.lambdatool'
         lambda_name = None
         try:
-            cwd = os.getcwd()
-            dirs = cwd.split('/')
-            lambda_name = dirs[-1]
+            with open(lambdatool, 'r') as j:
+                stuff = json.load(j)
+                lambda_name = stuff['name']
+
+            if not lambda_name:
+                cwd = os.getcwd()
+                dirs = cwd.split('/')
+                lambda_name = dirs[-1]
+
             logging.info('lambda_name: {}'.format(lambda_name))
         except Exception as x:
             logging.error('Exception caught in deploy_lambda(): {}'.format(x))
@@ -427,10 +449,12 @@ class LambdaDeployer:
 
     def find_data(self, the_dir):
         tree = []
+        package_file_name = (self._package_name.split('/'))[-1]
         try:
             for folder, subs, files in os.walk(the_dir):
                 for file in files:
-                    tree.append('{}/{}'.format(folder, file))
+                    if file != package_file_name:
+                        tree.append('{}/{}'.format(folder, file))
         except Exception:
             pass
 
@@ -486,3 +510,30 @@ class LambdaDeployer:
             logging.error('Exception caught in read_config_info(): {}'.format(wtf))
             traceback.print_exc(file=sys.stdout)
             return None
+
+    def _init_boto3_clients(self):
+        """
+        The utililty requires boto3 clients to SSM, Cloud Formation and S3. Here is
+        where we make them.
+
+        Args:
+            None
+
+        Returns:
+            Good or Bad; True or False
+        """
+        try:
+            if self._profile:
+                self._b3Sess = boto3.session.Session(profile_name=self._profile)
+            else:
+                self._b3Sess = boto3.session.Session()
+
+            self._s3_client = self._b3Sess.client('s3')
+            self._cf_client = self._b3Sess.client('cloudformation', region_name=self._region)
+            self._ssm_client = self._b3Sess.client('ssm', region_name=self._region)
+
+            return True
+        except Exception as wtf:
+            logging.error('Exception caught in intialize_session(): {}'.format(wtf))
+            traceback.print_exc(file=sys.stdout)
+            return False
